package org.springframework.security.openid.google;

import org.openid4java.consumer.ConsumerAssociationStore;
import org.openid4java.consumer.ConsumerManager;
import org.openid4java.consumer.InMemoryConsumerAssociationStore;
import org.openid4java.consumer.InMemoryNonceVerifier;
import org.openid4java.consumer.NonceVerifier;
import org.openid4java.discovery.Discovery;
import org.openid4java.discovery.html.HtmlResolver;
import org.openid4java.discovery.xri.XriDotNetProxyResolver;
import org.openid4java.discovery.xri.XriResolver;
import org.openid4java.discovery.yadis.YadisResolver;
import org.openid4java.server.RealmVerifierFactory;
import org.openid4java.util.HttpFetcherFactory;

import com.google.step2.ConsumerHelper;
import com.google.step2.discovery.DefaultHostMetaFetcher;
import com.google.step2.discovery.Discovery2;
import com.google.step2.discovery.HostMetaFetcher;
import com.google.step2.discovery.LegacyXrdsResolver;
import com.google.step2.discovery.XrdDiscoveryResolver;
import com.google.step2.http.DefaultHttpFetcher;
import com.google.step2.http.HttpFetcher;
import com.google.step2.xmlsimplesign.CachedCertPathValidator;
import com.google.step2.xmlsimplesign.DefaultCertValidator;
import com.google.step2.xmlsimplesign.DefaultTrustRootsProvider;
import com.google.step2.xmlsimplesign.TrustRootsProvider;
import com.google.step2.xmlsimplesign.Verifier;

public class DefaultConsumerHelperFactory implements ConsumerHelperFactory {
    
    private ConsumerAssociationStore consumerAssociationStore = new InMemoryConsumerAssociationStore();

    private NonceVerifier nonceVerifier = new InMemoryNonceVerifier(5000);
    
	public ConsumerAssociationStore getConsumerAssociationStore() {
		return consumerAssociationStore;
	}

	public void setConsumerAssociationStore(
			ConsumerAssociationStore consumerAssociationStore) {
		this.consumerAssociationStore = consumerAssociationStore;
	}

	public NonceVerifier getNonceVerifier() {
		return nonceVerifier;
	}

	public void setNonceVerifier(NonceVerifier nonceVerifier) {
		this.nonceVerifier = nonceVerifier;
	}

	public ConsumerHelper createConsumerHelper() {
		HttpFetcher httpFetcher = new DefaultHttpFetcher();
		
		HostMetaFetcher metaFetcher = new SerialHostMetaFetcher(
			new DefaultHostMetaFetcher(httpFetcher), 
			new GoogleHostedHostMetaFetcher(httpFetcher));

		HttpFetcherFactory httpFetcherFactory = new HttpFetcherFactory();
		
		TrustRootsProvider trustProvider = new DefaultTrustRootsProvider(); 
		Verifier verifier = new Verifier(new CachedCertPathValidator(trustProvider), httpFetcher);
		XrdDiscoveryResolver xrdDiscoveryResolver = 
			new LegacyXrdsResolver(httpFetcher, verifier, new DefaultCertValidator());
		
		HtmlResolver htmlResolver = new HtmlResolver(httpFetcherFactory);
		YadisResolver yadisResolver = new YadisResolver(httpFetcherFactory);
		XriResolver xriResolver = new XriDotNetProxyResolver(httpFetcherFactory);
		
		Discovery2 discovery2 = new Discovery2(
			metaFetcher, xrdDiscoveryResolver, htmlResolver, yadisResolver, xriResolver);
		RealmVerifierFactory verifierFactory = new RealmVerifierFactory(new YadisResolver(httpFetcherFactory));
		
		Discovery discovery = new Discovery(htmlResolver, yadisResolver, xriResolver);
		ConsumerManager consumerManager = 
			new ConsumerManager(verifierFactory, discovery, httpFetcherFactory);
		
		consumerManager.setNonceVerifier(nonceVerifier);
		consumerManager.setAssociations(consumerAssociationStore);
		
		return new ConsumerHelper(consumerManager, discovery2);		
	}
}
