/**
 * Copyright 2013 Rabbitsoft
 * Clinked
 */
package org.springframework.security.openid.memcache;

import java.io.IOException;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.spy.memcached.AddrUtil;
import net.spy.memcached.MemcachedClient;

import org.openid4java.consumer.AbstractNonceVerifier;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/** 
 * Nonce verifier with memcached store.
 * 
 * @author see AUTHORS.txt
 */
public class MemcacheNonceVerifier extends AbstractNonceVerifier {
	/** 
	 * {@link Logger}.
	 */
	private final Logger logger = LoggerFactory.getLogger(getClass());
	
	/** 
	 * List of memcached server addresses.
	 */
	private List<String> addressList;
	
	/*** 
	 * Returns memcached instance. 
	 * 
	 * @return client
	 * @throws IOException when fails
	 */
	private MemcachedClient getClient() throws IOException {
		return new MemcachedClient(AddrUtil.getAddresses(addressList));
	}
	
	/** 
	 * @param addressList {@link #addressList}
	 * @param maxAge how long the nonce should live
	 */
	public MemcacheNonceVerifier(List<String> addressList, int maxAge) {
		super(maxAge);
		
		this.addressList = addressList;
	}
	
	/** 
	 * @param addressList {@link #addressList}
	 * @param maxAge how long the nonce should live
	 */	
	public MemcacheNonceVerifier(String addressList, int maxAge) {
		this(Arrays.asList(addressList.split("(?:\\s|,)+")), maxAge);
	}	

	@SuppressWarnings("unchecked") @Override
	protected int seen(Date now, String opUrl, String nonce) {
		try {
			MemcachedClient client = getClient();
	        Set<String> seenSet = (Set<String>) client.get(opUrl);
	        if (seenSet == null) {
	            seenSet = new HashSet<String>();
	        }

	        if (seenSet.contains(nonce)) {
	            logger.error("Possible replay attack! Already seen nonce: " + nonce);
	            return SEEN;
	        }
	        seenSet.add(nonce);

	        logger.debug("Nonce verified: " + nonce);

	        client.add(opUrl, getMaxAge(), seenSet);
			client.shutdown();
			
	        return OK;
		} catch (Exception ex) {
			logger.error("Failed to verify nonce:", ex);
		}
		return OK;
	}
}
