/**
 * Copyright 2013 Rabbitsoft
 * Clinked
 */
package org.springframework.security.openid.redis;

import java.util.Collection;
import java.util.List;

import org.openid4java.association.Association;
import org.openid4java.consumer.ConsumerAssociationStore;

import redis.clients.jedis.JedisPoolConfig;

/** 
 * Stores all associations in memcached server. 
 * 
 * @author see AUTHORS.txt
 */
public class RedisConsumerAssociationStore implements ConsumerAssociationStore {

	private RedisHelper redisHelper;
	
	private final String hashPrefix = "RedisConsumerAssociationStore:";
	
	public RedisConsumerAssociationStore(JedisPoolConfig poolConfig, String host) {
		redisHelper = new RedisHelper(poolConfig, host);
	}

	@Override
	public void save(String opUrl, Association association) {
		removeExpired();
		redisHelper.hset(hashPrefix + opUrl, association.getHandle(), association);
	}

	@Override
	public Association load(String opUrl, String handle) {
		removeExpired();
		return (Association) redisHelper.hget(hashPrefix + opUrl, handle);
	}

	@Override
	public Association load(String opUrl) {
		removeExpired();
		
		List<Object> associations = redisHelper.hvals(hashPrefix + opUrl);
		
		Association latest = null;
		for (Object object : associations) {
			Association association = (Association) object;
			if (latest == null 
					|| latest.getExpiry().before(association.getExpiry())) {
                latest = association;			
			}
		}
		return latest;
	}

	@Override
	public void remove(String opUrl, String handle) {
		redisHelper.hdel(hashPrefix + opUrl, handle);
	}
	
	/** 
	 * Removes expired keys.
	 */
	private void removeExpired() {
		Collection<String> keySet = redisHelper.keys(hashPrefix); 
		for (String key : keySet) {
			for (Object object : redisHelper.hvals(key)) { 
				Association association = (Association) object;
				if (association.hasExpired()) {
					redisHelper.hdel(key, association.getHandle());
				}
			}
		}
	}
	
	public void destroy() {
		redisHelper.destroy();
	}
}
