/**
 * Copyright 2013 Rabbitsoft
 * Clinked
 */
package org.springframework.security.openid.redis;

import java.util.Date;

import org.openid4java.consumer.AbstractNonceVerifier;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import redis.clients.jedis.JedisPoolConfig;

/** 
 * Nonce verifier with memcached store.
 * 
 * @author see AUTHORS.txt
 */
public class RedisNonceVerifier extends AbstractNonceVerifier {
	/** 
	 * Prefix used in Redis store.
	 */
	private static final String PREFIX = "RedisNonceVerifier:";
	
	/** 
	 * {@link Logger}.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(RedisNonceVerifier.class);
	
	/** 
	 * {@link RedisHelper}.
	 */
	private RedisHelper redisHelper;
	
	
	/** 
	 * Initialize with pool config, host and max age.
	 * @param poolConfig pool config
	 * @param host host
	 * @param maxAge max age
	 */
	protected RedisNonceVerifier(JedisPoolConfig poolConfig, String host, int maxAge) {
		super(maxAge);
		redisHelper = new RedisHelper(poolConfig, host);
	}

	@Override
	protected int seen(Date now, String opUrl, String nonce) {
		String keyName = PREFIX + opUrl + ":" + nonce;
				
		Object seenNonce = redisHelper.get(keyName);
		if (seenNonce != null) {
			LOGGER.error("Possible replay attack! Already seen nonce: " + nonce);
			return SEEN;
		}
		
		redisHelper.set(keyName, now);
		redisHelper.expire(keyName, getMaxAge());
		
		return OK;
	}
	
	public void destroy() {
		redisHelper.destroy();
	}
	
}
